"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setLanguageByName = setLanguageByName;
exports.trModel = trModel;
exports.trField = trField;
exports.trError = trError;
Object.defineProperty(exports, "tr", {
  enumerable: true,
  get: function () {
    return _translate.tr;
  }
});
Object.defineProperty(exports, "setLanguage", {
  enumerable: true,
  get: function () {
    return _translate.setLanguage;
  }
});

var _translations = require("./translations");

var _translate = require("./translate");

var _utils = require("./utils");

const defLangs = {
  en: _translations.en,
  es: _translations.es
};
/**
 * Loads a language from the languages table.
 *
 * @param {string} name Language code as in `'en'` or `'fr'`.
 */

function setLanguageByName(name) {
  if (name in defLangs) (0, _translate.setLanguage)(defLangs[name]);
}
/**
 * Multipurpose semantic-ish translation.
 *
 * @param {string} modelName Object name, usually what
 *    you pass as the first parameter when you create
 *    the schema.
 * @param {string} field Field name
 * @param {string} op Thing that varies based on
 *    the type.
 */


function trModel(modelName, field, op) {
  return (0, _translate.tr)((0, _utils.trPath)(modelName, field, op));
}
/**
 * Translate field name
 *
 * @param {string|object} modelName Object name, usually what
 *    you pass as the first parameter when you create
 *    the schema. It can also be an object with component
 *    props so it will figure out the values
 * @param {string} field Field name
 */


function trField(modelName, field) {
  if (typeof modelName == 'object') {
    field = modelName.field;
    modelName = modelName.schemaTypeName;
  }

  return (0, _translate.tr)((0, _utils.trPath)(modelName, field, '_field'));
}
/**
 * Translates error message.
 *
 * @param {string} error Code of the error (usually the
 *    validation code-name)
 * @param {object} data Field configuration from `createSchema()`.
 */


function trError(error, data) {
  return (0, _translate.tr)(`error.${error}`, data);
}